﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OpenTK;
using ggj_2017.Elia;
using Aiv.Audio;

namespace ggj_2017.Elia {

    public abstract class TileManager{

        AudioClip audio;
        AudioSource player;

        int livelloMesso=0;

        private const int DISTANZA_DA_TILE = 250;

        protected List<int> posizioniCheckPoint;
        protected List<Tile> tileFisse;

        private List<Pair<Tile, float>> tilesDaTemporizzare = new List<Pair<Tile, float>>();
        private List<Tile> tilesAttuali = new List<Tile>();

        private float yPosPrecedente = Game.window.Height - 150;

        public TileManager() {
            audio = Sound.GetAudio("error");
            impostaPosizioniCheckPoint();
            impostaTileFisse();
            resettaLivello(yPosPrecedente);
        }

        public void aggiornaTile(float secondiPassati, float yPos) {
            if (!controllaSeLivelloCambiato(yPos)){
                if (!controllaSeResettareLivello(yPos) && !controllaFinito()){                   
                    aggiornaPiattaformeDaMostrare(secondiPassati, yPos);
                    aggiornaTempoPiattaformeAttuali(secondiPassati);
                } else {
                    resettaLivello(yPos);
                }
            }else {
                if (getLivello(yPos) < livelloMesso) {
                    resettaLivello(yPos);
                }
            }
            yPosPrecedente = yPos;
        }

        public List<Tile> getPiattaformeVisibili() {
            List<Tile> tiles = new List<Tile>();
            foreach(Tile tile in tileFisse) {
                tiles.Add(tile);
            }
            foreach (Tile tile in tilesAttuali) {
                tiles.Add(tile);
            }
            return tiles;
        }

        private bool controllaFinito() {
            if (tilesDaTemporizzare.Count <= 0 && tilesAttuali.Count<=0) {
                return true;
            }
            return false;
        }

        private bool controllaSeLivelloCambiato(float yPos) {
            return getLivello(yPos) != getLivello(yPosPrecedente);
        }

        private bool controllaSeResettareLivello(float yPos) {
            if (checkInsideCheck((int)yPos) && tilesAttuali.Count>0) {
                //Console.WriteLine(yPos);
                bool risultato = true;
                foreach(Tile tile in tilesAttuali) {
                    if (Math.Abs(yPos-tile.getY())<DISTANZA_DA_TILE) {
                        risultato = false;
                    }
                }
                if (risultato) {
                    player = new AudioSource();
                    player.Volume = 0.2f;
                    player.Play(audio);
                }
                return risultato;
            }
            return false;
        }

        private bool checkInsideCheck(int yPos) {
            bool risultato = false;
            foreach(int pos in posizioniCheckPoint) {
                if(yPos > pos-20 && yPos < pos + 20) {
                    risultato = true;
                }
            }
            return risultato;
        }

        private void aggiornaPiattaformeDaMostrare(float secondiPassati, float yPos) {
            foreach (Pair<Tile, float> tile in tilesDaTemporizzare) {
                tile.Second -= secondiPassati;
                if(tile.Second <= 0) {
                 
                    tilesAttuali.Add(tile.First);
                    tile.First.playNota();
                }
            }
            tilesDaTemporizzare.RemoveAll(tile => tile.Second <= 0);
        }

        private void aggiornaTempoPiattaformeAttuali(float secondiPassati) {
            foreach(Tile tile in tilesAttuali) {
                tile.diminuisciTempo(secondiPassati);
            }
            tilesAttuali.RemoveAll(tile => tile.tempoRimasto() <= 0);
        }

        public void resettaLivello(float yPos){
            tilesDaTemporizzare = getTilesLivello(getLivello(yPos));
            tilesAttuali = new List<Tile>();
            livelloMesso = getLivello(yPos);
        }

        protected abstract List<Pair<Tile, float>> getTilesLivello(int livello);

        protected abstract void impostaTileFisse();

        protected abstract void impostaPosizioniCheckPoint();
        protected abstract int getLivello(float yPos);

    }
}
