﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Aiv.Fast2D;
using OpenTK;
using Aiv.Audio;

namespace ggj_2017
{
    public class Rect
    {
        public Vector2 TopLeft;
        public Vector2 BottomRight;
        public Vector2 Offset;//Parent position
        public bool hittingDown;
        public bool hittingUp;
        public RigidBody Parent { get; set; }

        public float Width { get; protected set; }
        public float Height { get; protected set; }


        public float X
        {
            get
            {
                return Offset.X;
            }
        }

        public float Y
        {
            get
            {
                return Offset.Y;
            }
        }

        public float HalfWidth
        {
            get
            {
                return (Width / 2);
            }
        }


        public float HalfHeight
        {
            get
            {
                return (Height / 2);
            }
        }

        public Vector2 Center
        {
            get
            {
                return (new Vector2(Offset.X + HalfWidth, Offset.Y + HalfHeight));
            }
        }

        public Rect(Vector2 Top_Left, Vector2 Bottom_Right, RigidBody parentGeometry = null)
        {
            TopLeft = Top_Left;
            BottomRight = Bottom_Right;

            Width = BottomRight.X - TopLeft.X;
            Height = BottomRight.Y - TopLeft.Y;

            Parent = parentGeometry;
        }

        public Rect(Vector2 Top_Left, float width, float height, RigidBody parentGeometry = null) : this(Top_Left, new Vector2(Top_Left.X + width, Top_Left.Y + height), parentGeometry)
        {

        }

        public bool Contains(Vector2 point)
        {

            return (point.X >= TopLeft.X + Offset.X && point.X <= BottomRight.X + Offset.X) && (point.Y >= TopLeft.Y + Offset.Y && point.Y <= BottomRight.Y + Offset.Y);
        }


        //controlla collisione e riposiziona il rigid body nel posto giusto
        public bool Collide(Rect other)
        {
            //compute rectangles offset (the position of the parent)
            Offset = Parent.Parent.rigidBody.Position;
            other.Offset = other.Parent.Parent.rigidBody.Position;
            if (intersectAABBSat(this, other))
            {
                Parent.Parent.rigidBody.Position = Offset;
                //other.Parent.Parent.rigidBody.Position = other.Offset;
                
                return true;
            }
            else return false;
        }



        public static bool intersectAABBSat(Rect boxOne, Rect boxTwo)             //boxOne è quella che muoviamo noi
        {
            float distanceX = boxTwo.Center.X - boxOne.Center.X;
            float pointX = (boxOne.HalfWidth + boxTwo.HalfWidth) - Math.Abs(distanceX);
            if (pointX <= 0)
                return false;
            float distanceY = boxTwo.Center.Y - boxOne.Center.Y;
            float pointY = (boxOne.HalfHeight + boxTwo.HalfHeight) - Math.Abs(distanceY);
            if (pointY <= 0)
                return false;

            if (pointX < pointY)
            {
                if (distanceX > 0)
                {
                    boxOne.Offset.X = boxTwo.X - boxOne.Width;
                    //boxOne.hittingRight = true;
                }
                else
                {
                    boxOne.Offset.X = boxTwo.X + boxTwo.Width;
                    //boxOne.hittingLeft = true;
                }
            }
            else
            {
                if (distanceY > 0)
                {
                    boxOne.Offset.Y = boxTwo.Y - boxOne.Height;
                    boxOne.hittingDown = true;
                }
                else
                {
                    boxOne.Offset.Y = boxTwo.Y + boxTwo.Height;
                    boxOne.hittingUp = true;
                }
            }
            return true;
        }
    }
}
