﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Aiv.Fast2D;
using OpenTK;
using Aiv.Audio;

namespace ggj_2017
{
    public class RigidBody
    {
        public Vector2 Velocity { get; set; }
        public Vector2 Position { get; set; }
        public bool IsGravityAffected { get; set; }
        public Rect geometry { get; protected set; }

        public GameObject Parent { get; set; }

        public RigidBody(Vector2 position, GameObject parent, Rect boundingBox = null)
        {
            Position = position;
            Parent = parent;

            if (boundingBox != null)
            {
                geometry = boundingBox;
            }
            else
            {
                //default bounding box
                geometry = new Rect(new Vector2(0, 0), new Vector2(Parent.Width - 1, Parent.Height - 1));
                geometry.Parent = this;
            }

            PhysicsManager.AddBody(this);
        }

        public RigidBody(Vector2 position, GameObject parent, bool rigidForTile) {
            Position = position;
            Parent = parent;
            
            geometry = new Rect(new Vector2(0, 0), new Vector2(Parent.Width - 1, Parent.Height - 1));
            geometry.Parent = this;
        }

        public void AddVelocity(float velX, float velY)
        {
            Velocity = new Vector2(Velocity.X + velX, Velocity.Y + velY);
        }

        public void SetYvelocity(float vel)
        {
            Velocity = new Vector2(Velocity.X, vel);
        }

        public void SetXvelocity(float vel)
        {
            Velocity = new Vector2(vel, Velocity.Y);
        }

        public void Update()
        {
            if (IsGravityAffected)
                Velocity = new Vector2(Velocity.X, Velocity.Y + PhysicsManager.G);
            Position = new Vector2(Position.X + Velocity.X * Game.window.deltaTime, Position.Y + Velocity.Y * Game.window.deltaTime);
            //Position = new Vector2(Position.X + Velocity.X * 1f / 60, Position.Y + Velocity.Y * 1f / 60);
        }

        public bool isColliding(RigidBody other)
        {
            return this.geometry.Collide(other.geometry);
        }
    }
}
