﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Aiv.Fast2D;

namespace ggj_2017
{
    public class Input
    {
        Dictionary<KeyCode, KeyState> keys;
        KeyComboHandler comboHandler;

        public Input()
        {
            comboHandler = new KeyComboHandler();
            keys = new Dictionary<KeyCode, KeyState>();
            //prendo tutti i valori di keycode e li inserisco in un array
            Array array = Enum.GetValues(typeof(KeyCode));
            foreach (object item in array)
            {
                //downcasting
                KeyCode key = (KeyCode)item;
                keys.Add(key, new KeyState(key));
            }

        }

        internal void Draw()
        {
            comboHandler.Draw();
        }

        public void AddCombo(KeyCombo combo)
        {
            this.comboHandler.Add(combo);
        }

        public KeyCombo Match()     //Call it every Frame!
        {
            comboHandler.Update();
            foreach (var item in keys)
            {
                KeyCode key = item.Key;
                if (this.IsKeyDown(key))
                {
                    KeyCombo combo = comboHandler.Match(key);
                    if (combo != null)
                        return combo;
                }
            }
            return null;
        }


        public KeyState this[KeyCode key]
        {
            get
            {
                KeyState state = keys[key];
                state.Update();
                return state;
            }
        }

        public bool IsKeyPressed(KeyCode key)
        {
            KeyState state = keys[key];
            state.Update();
            return state.Pressed;
        }
        public bool IsKeyUp(KeyCode key)
        {
            KeyState state = keys[key];
            state.Update();
            return state.Up;
        }
        public bool IsKeyDown(KeyCode key)
        {
            KeyState state = keys[key];
            state.Update();
            return state.Down;
        }

    }
    public class KeyState
    {
        public bool Down { get; private set; }
        public bool Up { get; private set; }
        public bool Pressed { get; private set; }

        private KeyCode key;
        public KeyState(KeyCode key)
        {
            this.key = key;
        }

        public void Update()
        {
            if (Game.window.GetKey(key))
            {
                if (!Pressed)
                {
                    Down = true; //Entra in questa if nel Primo frame 
                }
                else
                {
                    Down = false; //Dal secondo frame in poi
                }
                Pressed = true;
            }
            else
            {
                if (Pressed)
                {
                    Up = true; //Entra in questa if nel Primo frame 
                }
                else
                {
                    Up = false;
                }
                Down = false;
                Pressed = false;
            }
        }
        public override string ToString()
        {
            return string.Format("key: {0}, Pressed: {1}, Down: {2}, Up: {3}", this.key, this.Pressed, this.Up, this.Down);
        }

    }

}
