﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Aiv.Fast2D;

namespace ggj_2017
{
    public class KeyComboHandler
    {
        List<KeyCombo> combos;
        public KeyComboHandler()
        {
            combos = new List<KeyCombo>();
        }

        public KeyCombo Match(KeyCode key)
        {
            for (int i = 0; i < combos.Count; i++)
            {
                KeyCombo keyCombo = combos[i];
                if (keyCombo.Match(key))
                {
                    ResetCombos();
                    return keyCombo;
                }
            }
            return null;
        }

        private void ResetCombos()
        {
            for (int i = 0; i < combos.Count; i++)
            {
                combos[i].Reset();
            }
        }

        internal void Draw()
        {
            foreach (var item in combos)
            {
                item.Draw();
            }
        }

        public void Add(KeyCombo combo)
        {
            combos.Add(combo);
        }

        public void Update()
        {
            for (int i = 0; i < combos.Count; i++)
            {
                KeyCombo combo = combos[i];
                if (combo.Active)
                    combo.Update();
            }
        }
    }

    public class KeyCombo
    {
        public bool Active { get; private set; }

        Queue<KeyCode> currentPressedKeys;
        KeyCode[] combo;
        float time;
        float currentTime;
        public KeyCombo(float time, params KeyCode[] keys)
        {
            currentPressedKeys = new Queue<KeyCode>();
            this.combo = keys;
            this.time = time;
        }

        public bool Match(KeyCode keyPressed)
        {
            this.currentPressedKeys.Enqueue(keyPressed);

            if(currentPressedKeys.Count == 1)
                StartTimer();

            bool match = true;
            for (int i = 0; i < currentPressedKeys.Count; i++)
            {
                KeyCode key = currentPressedKeys.Dequeue();
                match &= key == combo[i];
                if (!match)
                {
                    Reset();
                }
                else
                {
                    currentPressedKeys.Enqueue(key);
                }
            }

            if (this.currentPressedKeys.Count != this.combo.Length)
                return false;

            return true;
        }

        public void StartTimer()
        {
            Active = true;
            currentTime = time;
        }
        public void Reset()
        {
            Active = false;
            this.currentPressedKeys.Clear();
        }
        public void Update()
        {
            currentTime -= Game.window.deltaTime;
            if (currentTime <= 0)
                Reset();
        }

        public override string ToString()
        {
            string result = "Combo: ";
            for (int i = 0; i < combo.Length; i++)
            {
                if(i != combo.Length - 1)
                {
                    result += string.Format("{0} + ", combo[i]);
                }
                else
                {
                    result += string.Format("{0}", combo[i]);
                }
            }
            return result;
        }

        public void Draw()
        {
            Console.WriteLine(Active + " " + currentTime + ToString());
        }
      
    }
}
